@@ -60,7 +60,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/aux_/invariant_check.hpp"
 #include "libtorrent/aux_/session_impl.hpp"
 #include "libtorrent/aux_/file_view_pool.hpp"
-#include "libtorrent/disk_buffer_holder.hpp"
+#include "libtorrent/aux_/drive_info.hpp"
 #include "libtorrent/stat_cache.hpp"
 #include "libtorrent/hex.hpp" // to_hex
 
@@ -70,7 +70,7 @@ namespace libtorrent {
 
 namespace {
 
-error_code translate_error(std::system_error const& err, bool const write)
+error_code translate_error(std::error_code const& err, bool const write)
 {
 	// We don't really know why we failed to read or write. SIGBUS essentially
 	// means I/O failure. We assume that if we were writing, the failure was
@@ -78,10 +78,10 @@ error_code translate_error(std::system_error const& err, bool const write)
 	if (write)
 	{
 #ifdef TORRENT_WINDOWS
-		if (err.code() == std::error_code(sig::seh_errors::in_page_error))
+		if (err == std::error_code(sig::seh_errors::in_page_error))
 			return error_code(boost::system::errc::no_space_on_device, generic_category());
 #else
-		if (err.code() == std::error_code(sig::errors::bus))
+		if (err == std::error_code(sig::errors::bus))
 			return error_code(boost::system::errc::no_space_on_device, generic_category());
 #endif
 	}
@@ -89,14 +89,14 @@ error_code translate_error(std::system_error const& err, bool const write)
 #if BOOST_VERSION >= 107700
 
 #ifdef TORRENT_WINDOWS
-	if (err.code() == std::error_code(sig::seh_errors::in_page_error))
+	if (err == std::error_code(sig::seh_errors::in_page_error))
 		return error_code(boost::system::errc::io_error, generic_category());
 #else
-	if (err.code() == std::error_code(sig::errors::bus))
+	if (err == std::error_code(sig::errors::bus))
 		return error_code(boost::system::errc::io_error, generic_category());
 #endif
 
-	return err.code();
+	return err;
 #else
 
 	return error_code(boost::system::errc::io_error, generic_category());
@@ -164,7 +164,7 @@ error_code translate_error(std::system_error const& err, bool const write)
 			if (old_prio == dont_download && new_prio != dont_download)
 			{
 				// move stuff out of the part file
-				boost::optional<aux::file_view> f = open_file(sett, i, aux::open_mode::write, ec);
+				std::shared_ptr<aux::file_mapping> f = open_file(sett, i, aux::open_mode::write, ec);
 				if (ec)
 				{
 					prio = m_file_priority;
@@ -175,15 +175,22 @@ error_code translate_error(std::system_error const& err, bool const write)
 				{
 					try
 					{
-						m_part_file->export_file([&f](std::int64_t file_offset, span<char> buf)
+						m_part_file->export_file([&f](std::int64_t file_offset, span<char> buf) {
+							if (!f->has_memory_map())
 							{
+								lt::error_code err;
+								aux::pwrite_all(f->fd(), buf, file_offset, err);
+								if (err) throw lt::system_error(err);
+								return;
+							}
+
 							auto file_range = f->range().subspan(std::ptrdiff_t(file_offset));
 							TORRENT_ASSERT(file_range.size() >= buf.size());
 							sig::try_signal([&]{
 								std::memcpy(const_cast<char*>(file_range.data()), buf.data()
 									, static_cast<std::size_t>(buf.size()));
 								});
-							}, fs.file_offset(i), fs.file_size(i), ec.ec);
+						}, fs.file_offset(i), fs.file_size(i), ec.ec);
 
 						if (ec)
 						{
@@ -197,7 +204,14 @@ error_code translate_error(std::system_error const& err, bool const write)
 					{
 						ec.file(i);
 						ec.operation = operation_t::partfile_write;
-						ec.ec = translate_error(err, true);
+						ec.ec = translate_error(err.code(), true);
+						return;
+					}
+					catch (lt::system_error const& err)
+					{
+						ec.file(i);
+						ec.operation = operation_t::partfile_write;
+						ec.ec = err.code();
 						return;
 					}
 				}
@@ -290,14 +304,20 @@ error_code translate_error(std::system_error const& err, bool const write)
 	{
 		m_stat_cache.reserve(files().num_files());
 
-#ifdef TORRENT_WINDOWS
-		// don't do full file allocations on network drives
-		auto const file_name = convert_to_native_path_string(m_save_path);
-		int const drive_type = GetDriveTypeW(file_name.c_str());
-
-		if (drive_type == DRIVE_REMOTE)
+		auto const di = aux::get_drive_info(m_save_path);
+		if (di == aux::drive_info::remote)
+		{
+			// don't do full file allocations on network drives
 			m_allocate_files = false;
-#endif
+		}
+
+		switch (sett.get_int(settings_pack::disk_write_mode))
+		{
+			case settings_pack::always_pwrite: m_use_mmap_writes = false; break;
+			case settings_pack::always_mmap_write: m_use_mmap_writes = true; break;
+			case settings_pack::auto_mmap_write: m_use_mmap_writes = (di == aux::drive_info::ssd_dax); break;
+		}
+
 		{
 			std::unique_lock<std::mutex> l(m_file_created_mutex);
 			m_file_created.resize(files().num_files(), false);
@@ -396,7 +416,7 @@ error_code translate_error(std::system_error const& err, bool const write)
 			if (ec.ec)
 			{
 				ec.file(index);
-				ec.operation = operation_t::file_rename;
+				ec.operation = operation_t::mkdir;
 				return;
 			}
 
@@ -411,12 +431,11 @@ error_code translate_error(std::system_error const& err, bool const write)
 			if (ec)
 			{
 				ec.ec.clear();
-				copy_file(old_name, new_path, ec.ec);
+				aux::copy_file(old_name, new_path, ec);
 
 				if (ec)
 				{
 					ec.file(index);
-					ec.operation = operation_t::file_rename;
 					return;
 				}
 
@@ -428,7 +447,7 @@ error_code translate_error(std::system_error const& err, bool const write)
 		{
 			// if exists fails, report that error
 			ec.file(index);
-			ec.operation = operation_t::file_rename;
+			ec.operation = operation_t::file_stat;
 			return;
 		}
 
@@ -497,23 +516,23 @@ error_code translate_error(std::system_error const& err, bool const write)
 		return { ret, m_save_path };
 	}
 
-	int mmap_storage::readv(settings_interface const& sett
-		, span<iovec_t const> bufs
+	int mmap_storage::read(settings_interface const& sett
+		, span<char> buffer
 		, piece_index_t const piece, int const offset
 		, aux::open_mode_t const mode
 		, disk_job_flags_t const flags
 		, storage_error& error)
 	{
 #ifdef TORRENT_SIMULATE_SLOW_READ
-		std::this_thread::sleep_for(seconds(1));
+		std::this_thread::sleep_for(milliseconds(rand() % 2000));
 #endif
-		return readwritev(files(), bufs, piece, offset, error
+		return readwrite(files(), buffer, piece, offset, error
 			, [this, mode, flags, &sett](file_index_t const file_index
 				, std::int64_t const file_offset
-				, span<iovec_t const> vec, storage_error& ec)
+				, span<char> buf, storage_error& ec)
 		{
 			// reading from a pad file yields zeroes
-			if (files().pad_file_at(file_index)) return aux::read_zeroes(vec);
+			if (files().pad_file_at(file_index)) return aux::read_zeroes(buf);
 
 			if (file_index < m_file_priority.end_index()
 				&& m_file_priority[file_index] == dont_download
@@ -523,12 +542,11 @@ error_code translate_error(std::system_error const& err, bool const write)
 
 				error_code e;
 				peer_request map = files().map_file(file_index, file_offset, 0);
-				int const ret = m_part_file->readv(vec, map.piece, map.start, e);
+				int const ret = m_part_file->read(buf, map.piece, map.start, e);
 
 				if (e)
 				{
 					ec.ec = e;
-					ec.file(file_index);
 					ec.operation = operation_t::partfile_read;
 					return -1;
 				}
@@ -537,54 +555,48 @@ error_code translate_error(std::system_error const& err, bool const write)
 
 			auto handle = open_file(sett, file_index, mode, ec);
 			if (ec) return -1;
+			TORRENT_ASSERT(handle);
+
+			if (!handle->has_memory_map())
+				return aux::pread_all(handle->fd(), buf, file_offset, ec.ec);
 
 			int ret = 0;
-			error_code e;
 			span<byte const> file_range = handle->range();
 
 			// set this unconditionally in case the upper layer would like to treat
 			// short reads as errors
 			ec.operation = operation_t::file_read;
 
+			if (file_range.size() <= file_offset)
+			{
+				ec.ec = boost::asio::error::eof;
+				return -1;
+			}
+
 			try
 			{
-				if (file_range.size() > file_offset)
+				file_range = file_range.subspan(static_cast<std::ptrdiff_t>(file_offset));
+				if (!file_range.empty())
 				{
-					file_range = file_range.subspan(static_cast<std::ptrdiff_t>(file_offset));
-					for (auto buf : vec)
-					{
-						if (file_range.empty()) break;
-						if (file_range.size() < buf.size()) buf = buf.first(file_range.size());
+					if (file_range.size() < buf.size()) buf = buf.first(file_range.size());
 
-						sig::try_signal([&]{
-							std::memcpy(buf.data(), const_cast<char*>(file_range.data())
-								, static_cast<std::size_t>(buf.size()));
-							});
+					sig::try_signal([&]{
+						std::memcpy(buf.data(), const_cast<char*>(file_range.data())
+							, static_cast<std::size_t>(buf.size()));
+						});
 
-						file_range = file_range.subspan(buf.size());
-						ret += static_cast<int>(buf.size());
-					}
 					if (flags & disk_interface::volatile_read)
-						handle->dont_need(file_range);
+						handle->dont_need(file_range.first(buf.size()));
 					if (flags & disk_interface::flush_piece)
-						handle->page_out(file_range);
+						handle->page_out(file_range.first(buf.size()));
+
+					file_range = file_range.subspan(buf.size());
+					ret += static_cast<int>(buf.size());
 				}
 			}
 			catch (std::system_error const& err)
 			{
-				ec.file(file_index);
-				ec.ec = translate_error(err, false);
-				return -1;
-			}
-
-			// we either get an error or 0 or more bytes read
-			TORRENT_ASSERT(e || ret > 0);
-			TORRENT_ASSERT(ret <= bufs_size(vec));
-
-			if (e)
-			{
-				ec.ec = e;
-				ec.file(file_index);
+				ec.ec = translate_error(err.code(), false);
 				return -1;
 			}
 
@@ -592,22 +604,25 @@ error_code translate_error(std::system_error const& err, bool const write)
 		});
 	}
 
-	int mmap_storage::writev(settings_interface const& sett
-		, span<iovec_t const> bufs
+	int mmap_storage::write(settings_interface const& sett
+		, span<char> buffer
 		, piece_index_t const piece, int const offset
 		, aux::open_mode_t const mode
 		, disk_job_flags_t const flags
 		, storage_error& error)
 	{
-		return readwritev(files(), bufs, piece, offset, error
+#ifdef TORRENT_SIMULATE_SLOW_WRITE
+		std::this_thread::sleep_for(milliseconds(rand() % 800));
+#endif
+		return readwrite(files(), buffer, piece, offset, error
 			, [this, mode, flags, &sett](file_index_t const file_index
 				, std::int64_t const file_offset
-				, span<iovec_t const> vec, storage_error& ec)
+				, span<char> buf, storage_error& ec)
 		{
 			if (files().pad_file_at(file_index))
 			{
 				// writing to a pad-file is a no-op
-				return bufs_size(vec);
+				return int(buf.size());
 			}
 
 			if (file_index < m_file_priority.end_index()
@@ -619,12 +634,11 @@ error_code translate_error(std::system_error const& err, bool const write)
 				error_code e;
 				peer_request map = files().map_file(file_index
 					, file_offset, 0);
-				int const ret = m_part_file->writev(vec, map.piece, map.start, e);
+				int const ret = m_part_file->write(buf, map.piece, map.start, e);
 
 				if (e)
 				{
 					ec.ec = e;
-					ec.file(file_index);
 					ec.operation = operation_t::partfile_write;
 					return -1;
 				}
@@ -635,41 +649,41 @@ error_code translate_error(std::system_error const& err, bool const write)
 			// we're writing to it
 			m_stat_cache.set_dirty(file_index);
 
+			TORRENT_ASSERT(file_index < m_files.end_file());
+
 			auto handle = open_file(sett, file_index
 				, aux::open_mode::write | mode, ec);
 			if (ec) return -1;
 
-			int ret = 0;
-			error_code e;
-			span<byte> file_range = handle->range().subspan(static_cast<std::ptrdiff_t>(file_offset));
-
 			// set this unconditionally in case the upper layer would like to treat
 			// short reads as errors
 			ec.operation = operation_t::file_write;
 
+			if (!m_use_mmap_writes || !handle->has_memory_map())
+				return aux::pwrite_all(handle->fd(), buf, file_offset, ec.ec);
+
+			int ret = 0;
+			span<byte> file_range = handle->range().subspan(static_cast<std::ptrdiff_t>(file_offset));
+
 			try
 			{
-				for (auto buf : vec)
-				{
-					TORRENT_ASSERT(file_range.size() >= buf.size());
+				TORRENT_ASSERT(file_range.size() >= buf.size());
 
-					sig::try_signal([&]{
-						std::memcpy(const_cast<char*>(file_range.data()), buf.data(), static_cast<std::size_t>(buf.size()));
-						});
+				sig::try_signal([&]{
+					std::memcpy(const_cast<char*>(file_range.data()), buf.data(), static_cast<std::size_t>(buf.size()));
+					});
 
-					file_range = file_range.subspan(buf.size());
-					ret += static_cast<int>(buf.size());
-				}
+				file_range = file_range.subspan(buf.size());
+				ret += static_cast<int>(buf.size());
 
 				if (flags & disk_interface::volatile_read)
-					handle->dont_need(file_range);
+					handle->dont_need(file_range.first(buf.size()));
 				if (flags & disk_interface::flush_piece)
-					handle->page_out(file_range);
+					handle->page_out(file_range.first(buf.size()));
 			}
 			catch (std::system_error const& err)
 			{
-				ec.file(file_index);
-				ec.ec = translate_error(err, true);
+				ec.ec = translate_error(err.code(), true);
 				return -1;
 			}
 
@@ -677,18 +691,11 @@ error_code translate_error(std::system_error const& err, bool const write)
 			m_pool.record_file_write(storage_index(), file_index, ret);
 #endif
 
-			if (e)
-			{
-				ec.ec = e;
-				ec.file(file_index);
-				return -1;
-			}
-
 			return ret;
 		});
 	}
 
-	int mmap_storage::hashv(settings_interface const& sett
+	int mmap_storage::hash(settings_interface const& sett
 		, hasher& ph, std::ptrdiff_t const len
 		, piece_index_t const piece, int const offset
 		, aux::open_mode_t const mode
@@ -696,30 +703,19 @@ error_code translate_error(std::system_error const& err, bool const write)
 		, storage_error& error)
 	{
 #ifdef TORRENT_SIMULATE_SLOW_READ
-		std::this_thread::sleep_for(seconds(1));
+		std::this_thread::sleep_for(milliseconds(rand() % 2000));
 #endif
-		char dummy = 0;
-		iovec_t dummy1 = {&dummy, len};
-		span<iovec_t> dummy2(&dummy1, 1);
 
-		return readwritev(files(), dummy2, piece, offset, error
-			, [this, mode, flags, &ph, &sett](file_index_t const file_index
+		char dummy;
+		std::vector<char> scratch;
+
+		return readwrite(files(), {&dummy, len}, piece, offset, error
+			, [this, mode, flags, &ph, &sett, &scratch](file_index_t const file_index
 				, std::int64_t const file_offset
-				, span<iovec_t const> vec, storage_error& ec)
+				, span<char> const buf, storage_error& ec)
 		{
-			auto const read_size = bufs_size(vec);
-
 			if (files().pad_file_at(file_index))
-			{
-				std::array<char, 64> zero_buf;
-				zero_buf.fill(0);
-				span<char> zeroes = zero_buf;
-				for (std::ptrdiff_t left = read_size; left > 0; left -= zeroes.size())
-				{
-					ph.update({zeroes.data(), std::min(zeroes.size(), left)});
-				}
-				return read_size;
-			}
+				return aux::hash_zeroes(ph, buf.size());
 
 			if (file_index < m_file_priority.end_index()
 				&& m_file_priority[file_index] == dont_download
@@ -727,13 +723,12 @@ error_code translate_error(std::system_error const& err, bool const write)
 			{
 				error_code e;
 				peer_request map = files().map_file(file_index, file_offset, 0);
-				int const ret = m_part_file->hashv(ph, read_size
+				int const ret = m_part_file->hash(ph, buf.size()
 					, map.piece, map.start, e);
 
 				if (e)
 				{
 					ec.ec = e;
-					ec.file(file_index);
 					ec.operation = operation_t::partfile_read;
 					return -1;
 				}
@@ -743,12 +738,21 @@ error_code translate_error(std::system_error const& err, bool const write)
 			auto handle = open_file(sett, file_index, mode, ec);
 			if (ec) return -1;
 
+			if (!handle->has_memory_map())
+			{
+				scratch.resize(std::size_t(buf.size()));
+				int const ret = aux::pread_all(handle->fd(), scratch, file_offset, ec.ec);
+				if (ec) return -1;
+				ph.update(scratch);
+				return ret;
+			}
+
 			int ret = 0;
 			span<byte const> file_range = handle->range();
 			if (file_range.size() > file_offset)
 			{
 				file_range = file_range.subspan(std::ptrdiff_t(file_offset)
-					, std::min(std::ptrdiff_t(read_size), std::ptrdiff_t(file_range.size() - file_offset)));
+					, std::min(buf.size(), std::ptrdiff_t(file_range.size() - file_offset)));
 
 				sig::try_signal([&]{
 					ph.update({const_cast<char const*>(file_range.data()), file_range.size()});
@@ -764,7 +768,7 @@ error_code translate_error(std::system_error const& err, bool const write)
 		});
 	}
 
-	int mmap_storage::hashv2(settings_interface const& sett
+	int mmap_storage::hash2(settings_interface const& sett
 		, hasher256& ph, std::ptrdiff_t const len
 		, piece_index_t const piece, int const offset
 		, aux::open_mode_t const mode
@@ -783,7 +787,7 @@ error_code translate_error(std::system_error const& err, bool const write)
 		{
 			error_code e;
 			peer_request map = files().map_file(file_index, file_offset, 0);
-			int const ret = m_part_file->hashv2(ph, len
+			int const ret = m_part_file->hash2(ph, len
 				, map.piece, map.start, e);
 
 			if (e)
@@ -799,9 +803,28 @@ error_code translate_error(std::system_error const& err, bool const write)
 		auto handle = open_file(sett, file_index, mode, error);
 		if (error) return -1;
 
+		if (!handle->has_memory_map())
+		{
+			std::vector<char> scratch(static_cast<std::size_t>(len));
+			int const ret = aux::pread_all(handle->fd(), scratch, file_offset, error.ec);
+			if (error)
+			{
+				error.file(file_index);
+				error.operation = operation_t::file_read;
+				return -1;
+			}
+			ph.update(scratch);
+			return ret;
+		}
+
 		span<byte const> file_range = handle->range();
 		if (std::int64_t(file_range.size()) <= file_offset)
-			return 0;
+		{
+			error.ec = boost::asio::error::eof;
+			error.file(file_index);
+			error.operation = operation_t::file_read;
+			return -1;
+		}
 		file_range = file_range.subspan(std::ptrdiff_t(file_offset));
 		file_range = file_range.first(std::min(std::ptrdiff_t(len), file_range.size()));
 		ph.update(file_range);
@@ -815,7 +838,7 @@ error_code translate_error(std::system_error const& err, bool const write)
 
 	// a wrapper around open_file_impl that, if it fails, makes sure the
 	// directories have been created and retries
-	boost::optional<aux::file_view> mmap_storage::open_file(settings_interface const& sett
+	std::shared_ptr<aux::file_mapping> mmap_storage::open_file(settings_interface const& sett
 		, file_index_t const file
 		, aux::open_mode_t mode, storage_error& ec) const
 	{
@@ -844,34 +867,7 @@ error_code translate_error(std::system_error const& err, bool const write)
 		}
 #endif
 
-		boost::optional<aux::file_view> h = open_file_impl(sett, file, mode, ec);
-		if ((mode & aux::open_mode::write)
-			&& (ec.ec == boost::system::errc::no_such_file_or_directory
-#ifdef TORRENT_WINDOWS
-				// this is a workaround for improper handling of files on windows shared drives.
-				// if the directory on a shared drive does not exist,
-				// windows returns ERROR_IO_DEVICE instead of ERROR_FILE_NOT_FOUND
-				|| ec.ec == error_code(ERROR_IO_DEVICE, system_category())
-#endif
-		))
-		{
-			// this means the directory the file is in doesn't exist.
-			// so create it
-			ec.ec.clear();
-			std::string path = files().file_path(file, m_save_path);
-			create_directories(parent_path(path), ec.ec);
-
-			if (ec.ec)
-			{
-				// if the directory creation failed, don't try to open the file again
-				// but actually just fail
-				ec.file(file);
-				ec.operation = operation_t::mkdir;
-				return {};
-			}
-
-			h = open_file_impl(sett, file, mode, ec);
-		}
+		std::shared_ptr<aux::file_mapping> h = open_file_impl(sett, file, mode, ec);
 		if (ec.ec)
 		{
 			ec.file(file);
@@ -892,7 +888,7 @@ error_code translate_error(std::system_error const& err, bool const write)
 		return h;
 	}
 
-	boost::optional<aux::file_view> mmap_storage::open_file_impl(settings_interface const& sett
+	std::shared_ptr<aux::file_mapping> mmap_storage::open_file_impl(settings_interface const& sett
 		, file_index_t file
 		, aux::open_mode_t mode
 		, storage_error& ec) const
@@ -905,9 +901,11 @@ error_code translate_error(std::system_error const& err, bool const write)
 			mode |= aux::open_mode::sparse;
 
 		if (sett.get_bool(settings_pack::no_atime_storage))
-		{
 			mode |= aux::open_mode::no_atime;
-		}
+
+		if (files().file_size(file) / default_block_size
+			<= sett.get_int(settings_pack::mmap_file_size_cutoff))
+			mode |= aux::open_mode::no_mmap;
 
 		// if we have a cache already, don't store the data twice by leaving it in the OS cache as well
 		auto const write_mode = sett.get_int(settings_pack::disk_io_write_mode);
