@@ -17,7 +17,7 @@
  * All dynamically allocated memory comes from the stack.  The stack required
  * is less than 2K bytes.  This code is compatible with 16-bit int's and
  * assumes that long's are at least 32 bits.  puff.c uses the short data type,
- * assumed to be 16 bits, for arrays in order to to conserve memory.  The code
+ * assumed to be 16 bits, for arrays in order to conserve memory.  The code
  * works whether integers are stored big endian or little endian.
  *
  * In the comments below are "Format notes" that describe the inflate process
@@ -82,8 +82,8 @@
 // this whole file is just preserved and warnings are suppressed
 #include "libtorrent/aux_/disable_warnings_push.hpp"
 
-#include <setjmp.h>             /* for setjmp(), longjmp(), and jmp_buf */
-#include <string.h>             /* for NULL */
+#include <csetjmp>             /* for setjmp(), longjmp(), and jmp_buf */
+#include <cstring>             /* for nullptr */
 #include "libtorrent/puff.hpp"             /* prototype for puff() */
 
 #define local static            /* for local function definitions */
@@ -113,7 +113,7 @@ struct state {
     int bitcnt;                 /* number of bits in bit buffer */
 
     /* input limit error return state for bits() and decode() */
-    jmp_buf env;
+	 std::jmp_buf env;
 };
 
 /*
@@ -135,7 +135,7 @@ local int bits(struct state *s, int need)
     val = s->bitbuf;
     while (s->bitcnt < need) {
         if (s->incnt == s->inlen)
-            longjmp(s->env, 1);         /* out of input */
+            std::longjmp(s->env, 1);         /* out of input */
         val |= long(s->in[s->incnt++]) << s->bitcnt;  /* load eight bits */
         s->bitcnt += 8;
     }
@@ -185,7 +185,7 @@ local int stored(struct state *s)
     /* copy len bytes from in to out */
     if (s->incnt + len > s->inlen)
         return 2;                               /* not enough input */
-    if (s->out != NULL) {
+    if (s->out != nullptr) {
         if (s->outcnt + len > s->outlen)
             return 1;                           /* not enough output space */
         while (len--)
@@ -280,7 +280,7 @@ local int decode(struct state *s, const struct huffman *h)
     code = first = index = 0;
     len = 1;
     next = h->count + 1;
-    while (1) {
+    for (;;) {
         while (left--) {
             code |= bitbuf & 1;
             bitbuf >>= 1;
@@ -300,7 +300,7 @@ local int decode(struct state *s, const struct huffman *h)
         if (left == 0)
             break;
         if (s->incnt == s->inlen)
-            longjmp(s->env, 1);         /* out of input */
+            std::longjmp(s->env, 1);         /* out of input */
         bitbuf = s->in[s->incnt++];
         if (left > 8)
             left = 8;
@@ -466,7 +466,7 @@ local int codes(struct state *s,
             return symbol;              /* invalid symbol */
         if (symbol < 256) {             /* literal: symbol is the byte */
             /* write out the literal */
-            if (s->out != NULL) {
+            if (s->out != nullptr) {
                 if (s->outcnt == s->outlen)
                     return 1;
                 s->out[s->outcnt] = symbol;
@@ -491,7 +491,7 @@ local int codes(struct state *s,
 #endif
 
             /* copy length bytes from distance bytes back */
-            if (s->out != NULL) {
+            if (s->out != nullptr) {
                 if (s->outcnt + len > s->outlen)
                     return 1;
                 while (len--) {
@@ -628,7 +628,7 @@ local int fixed(struct state *s)
  *   are themselves compressed using Huffman codes and run-length encoding.  In
  *   the list of code lengths, a 0 symbol means no code, a 1..15 symbol means
  *   that length, and the symbols 16, 17, and 18 are run-length instructions.
- *   Each of 16, 17, and 18 are follwed by extra bits to define the length of
+ *   Each of 16, 17, and 18 are followed by extra bits to define the length of
  *   the run.  16 copies the last length 3 to 6 times.  17 represents 3 to 10
  *   zero lengths, and 18 represents 11 to 138 zero lengths.  Unused symbols
  *   are common, hence the special coding for zero lengths.
@@ -706,7 +706,6 @@ local int dynamic(struct state *s)
     index = 0;
     while (index < nlen + ndist) {
         int symbol;             /* decoded value */
-        int len;                /* last length to repeat */
 
         symbol = decode(s, &lencode);
         if (symbol < 0)
@@ -714,7 +713,8 @@ local int dynamic(struct state *s)
         if (symbol < 16)                /* length in 0..15 */
             lengths[index++] = symbol;
         else {                          /* repeat instruction */
-            len = 0;                    /* assume repeating zeros */
+            int len = 0;                /* last length to repeat */
+                                        /* assume repeating zeros */
             if (symbol == 16) {         /* repeat last length 3..6 times */
                 if (index == 0)
                     return -5;          /* no last length! */
